/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Copyright (C) 2008 University of Waikato 
 */

package weka.filters.unsupervised.instance;

import weka.core.Instance;
import weka.core.Instances;
import weka.filters.AbstractFilterTest;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.ReplaceMissingValues;

import java.util.Random;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * Tests IdenticalInstanceFilter. Run from the command line with:<p>
 * java weka.filters.unsupervised.instance.IdenticalInstanceFilterTest
 *
 * @author FracPete (fracpete at waikato dot ac dot nz)
 * @version $Revision$
 */
public class IdenticalInstanceFilterTest extends AbstractFilterTest {

  /** the number of duplicate instances. */
  public final static int NUM_DUPLICATES = 5;
  
  /**
   * Initializes the test.
   * 
   * @param name	the name of the test
   */
  public IdenticalInstanceFilterTest(String name) { 
    super(name);
  }

  /**
   * Creates a default IdenticalInstanceFilter.
   * 
   * @return		the filter
   */
  public Filter getFilter() {
    return new IdenticalInstanceFilter();
  }

  /**
   * Initializes the test.
   * 
   * @throws Exception 	if something goes wrong
   */
  protected void setUp() throws Exception {
    ReplaceMissingValues	replace;
    Random			random;
    int				i;
    
    super.setUp();

    // remove string attributes
    m_Instances.deleteStringAttributes();
    
    // replace missing values
    replace = new ReplaceMissingValues();
    replace.setInputFormat(m_Instances);
    m_Instances = Filter.useFilter(m_Instances, replace);
    
    // duplicate some instances
    random = new Random(1);
    for (i = 0; i < NUM_DUPLICATES; i++) {
      m_Instances.add(
	    (Instance) m_Instances.instance(
		  random.nextInt(m_Instances.numInstances() - i)).copy());
    }
  }

  /**
   * Tests the merging of the instances.
   */
  public void testMerging() {
    ((IdenticalInstanceFilter) m_Filter).setMerge(true);
    Instances result = useFilter();
    assertEquals(m_Instances.numAttributes(), result.numAttributes());
    assertEquals(m_Instances.numInstances() - NUM_DUPLICATES,  result.numInstances());
  }

  /**
   * Tests the deletion of the instances.
   */
  public void testDeleting() {
    Instances result = useFilter();
    assertEquals(m_Instances.numAttributes(), result.numAttributes());
    assertEquals(m_Instances.numInstances() - NUM_DUPLICATES,  result.numInstances());
  }

  /**
   * Returns a test suite.
   * 
   * @return		the suite
   */
  public static Test suite() {
    return new TestSuite(IdenticalInstanceFilterTest.class);
  }

  /**
   * Runs the test from commandline.
   * 
   * @param args	ignored
   */
  public static void main(String[] args){
    junit.textui.TestRunner.run(suite());
  }
}
